<?php
/**
 * Основной контроллер приложения
 *
 * @version 1.0
 * @link http://www.simplecoding.org
 * @author Стаценко Владимир <vova_33@gala.net>
 */
class BugTracker extends Controller {
	
	//настройки отображения списка багов
	//добавляем CSS классы, параметры в фигурных скобках будут заменены на
	//соответствующие значения шаблонизатором
	private $listConf = array(
		'commentOpen'=>'<li class="depth-{depth}">'
		,'commentOpen' => '<li class="commentnode commentnode-{depth}">'
	);

	/**
	 * Конструктор
	 * Здесь выполняется загрузка библиотек и моделей
	 */
	function BugTracker() {
		parent::Controller();

		$this->load->model('mbug');
		$this->load->model('mcategory');
		$this->load->model('mcomment');
		$this->load->library('Table2Tree');
		$this->load->helper('form');
		$this->load->language('form_fields');
	}

	/**
	 * Формирует главную страницу баг трекера. Совпадает
	 * с первой страницей списка багов.
	 * @return html страница
	 */
	function index() {
		$this->page();
	}
	
	/**
	 * Формирует страницу со списком багов.
	 * 
	 * @param $firstBug - номер первого бага на странице
	 * @return html страница
	 */
	function page($firstBug = 0) {
		if (!is_integer((int)$firstBug)) {
			$firstBug = 0;
		}
		
		//сохраняем адрес этой страницы в сессии
		$this->session->set_userdata(array('prev_page'=>current_url()));
		
		$pageData['title'] = 'Bug Tracker';

		//настраиваем разбивку на страницы
		$this->load->library('pagination');
		$pconf['base_url'] = $this->config->item('base_url').'bugtracker/page';
		$pconf['total_rows'] = $this->mbug->getBugsCount();
		$pconf['per_page'] = $this->config->item('bugs_per_page');
		
		$this->pagination->initialize($pconf);
		$pageData['paginationLinks'] = $this->pagination->create_links();

		//загружаем общий список ошибок и комментариев к ним
		$bugs = $this->mbug->getAllBugs($firstBug, $this->config->item('bugs_per_page'));
		
		if ($bugs !== false) {
			$bugsTree = $this->table2tree->getTree($bugs, $firstBug, $this->config->item('bugs_per_page'));
			
			$pageData['bugsList'] = $this->table2tree->getHTMLList($bugsTree, $this->listConf);
			
		}

		$pageData['categories'] = $this->mcategory->getAllCategories();
		
		$this->load->view('header', $pageData);
		$this->load->view('categories');
		//размещаем форму отправки сообщений об ошибках
		$this->load->view('addbugform');
		$this->load->view('allbugs');
		$this->load->view('footer');
	}
	
	/**
	 * Формирует страницу со списком багов выбранной категории.
	 * 
	 * @param $categoryLink - ссылка на категорию
	 * @param $firstBug - номер первого бага в списке
	 * @return html страница
	 */
	function category($categoryLink, $firstBug = 0) {
		if (!is_integer((int)$firstBug)) {
			$firstBug = 0;
		}

		//сохраняем адрес этой страницы в сессии
		$this->session->set_userdata(array('prev_page'=>current_url()));
		
		$pageData['title'] = 'Bug Tracker | Category';

		//настраиваем разбивку на страницы
		$this->load->library('pagination');
		$pconf['base_url'] = $this->config->item('base_url').'bugtracker/category/'.$categoryLink;
		$pconf['total_rows'] = $this->mbug->getBugsCountByCategory($categoryLink);
		$pconf['per_page'] = $this->config->item('bugs_per_page');
		$pconf['uri_segment'] = 4;

		$this->pagination->initialize($pconf);
		$pageData['paginationLinks'] = $this->pagination->create_links();
		
		//загружаем общий список багов для выбранной категории
		$bugs = $this->mbug->getBugsByCategory($categoryLink, $firstBug, $this->config->item('bugs_per_page'));
		
		if ($bugs !== false) {
			$bugsTree = $this->table2tree->getTree($bugs);
			$pageData['bugsList'] = $this->table2tree->getHTMLList($bugsTree, $this->listConf);
		}

		$pageData['categories'] = $this->mcategory->getAllCategories();

		$this->load->view('header', $pageData);
		$this->load->view('categories');
		//размещаем форму отправки сообщений об ошибках
		$this->load->helper('form');
		$this->load->view('addbugform');
		$this->load->view('categorybugs');
		$this->load->view('footer');
	}
	
	/**
	 * Показывает страницу с подробной информацией о выбранном баге (описание, комментарии и т.п.)
	 * 
	 * @param $id - id бага в базе данных
	 * @return html страница
	 */
	function bug($id = 1) {
		if (!is_integer((int)$id) || (int)$id <= 0) {
			redirect('bugtracker/index');
			return;
		}
		
		//сохраняем адрес этой страницы в сессии
		$this->session->set_userdata(array('prev_page'=>current_url()));
		
		$pageData['title'] = 'Bugtracker';
		
		//ищем указанный баг
		$bug = $this->mbug->getBug($id);
		if ($bug !== false) {
			$bugTree = $this->table2tree->getTree($bug);
			
			$pageData['bugList'] = $this->table2tree->getHTMLList($bugTree, $this->listConf);
			$pageData['bug_id'] = $bugTree[0]['id'];
		}
		
		$pageData['categories'] = $this->mcategory->getAllCategories();
		
		$this->load->view('header', $pageData);
		$this->load->view('categories');
		$this->load->view('onebug');
		if ($bug !== false) {
			$this->load->view('addcommentform');
		}
		$this->load->view('footer');
	}
	
	/**
	 * Сохраняет в БД запись о новом баге. Данные приходят в массиве $_POST.
	 * @return в случае успеха - редирект на последнюю страницу с которой было
	 * отправлено сообщение о баге, в случае неудачи - html страница с формой
	 * и описаниями ошибок. 
	 */
	function addbug() {
		$this->load->library('form_validation');
		$this->form_validation->set_error_delimiters('<div class="fErrMessage">', '</div>');
		$this->form_validation->set_rules('title', 'lang:title', 'required');
		$this->form_validation->set_rules('uname', 'lang:uname', 'required');
		$this->form_validation->set_rules('category_id', 'lang:category_id', 'required|integer');
		$this->form_validation->set_rules('description', 'lang:description', 'required');
		$this->form_validation->set_rules('email', 'lang:email', 'required|valid_email');
		if ($this->form_validation->run() === FALSE) {
			//ошибка
			$pageData['message'] = 'Форма заполнена неправильно';
			
			//показываем форму с описаниями ошибок
			$pageData['title'] = 'Bug Tracker';
			$pageData['categories'] = $this->mcategory->getAllCategories();
			
			$this->load->view('header', $pageData);
			$this->load->view('categories');
			$this->load->view('addbugform');
			$this->load->view('footer');
		}
		else {
			//форма заполнена правильно
			$this->load->library('htmlpurifier/HTMLPurifier');
			$config = HTMLPurifier_Config::createDefault();
			
			$bugData['title'] = $this->htmlpurifier->purify($this->input->post('title'));
			$bugData['uname'] = $this->htmlpurifier->purify($this->input->post('uname'));
			$bugData['category_id'] = $this->input->post('category_id');
			$bugData['uemail'] = $this->input->post('email');
			$bugData['description'] = $this->htmlpurifier->purify($this->input->post('description'));

			if (($res = $this->mbug->addNewBug($bugData)) !== FALSE) {
				//данные добавлены
				$this->session->set_flashdata('message', 'Сообщение добавлено');
			}
			else {
				//при добавлении данных возникла ошибка
				$this->session->set_flashdata('message', 'При добавлении данных возникла ошибка');
			}
			
			redirect($this->session->userdata('prev_page'));
		}
	}
	
	/**
	 * Сохраняет в БД запись о новом комментарии. Данные приходят в массиве $_POST.
	 * @return в случае успеха - редирект на последнюю страницу с которой был
	 * отправлен комментарий, в случае неудачи - html страница с формой
	 * и описаниями ошибок. 
	 */
	function addcomment() {
		$this->load->library('form_validation');
		$this->form_validation->set_error_delimiters('<div class="fErrMessage">', '</div>');
		$this->form_validation->set_rules('uname', 'lang:uname', 'required');
		$this->form_validation->set_rules('description', 'lang:description', 'required');
		$this->form_validation->set_rules('email', 'lang:email', 'required|valid_email');
		$this->form_validation->set_rules('parent_id', 'lang:parent_id', 'integer');
		if ($this->form_validation->run() === FALSE) {
			//ошибка
			$pageData['message'] = 'Форма заполнена неправильно';
			
			//показываем форму с описаниями ошибок
			$pageData['title'] = 'Bug Tracker';
			$pageData['bug_id'] = $this->input->post('bug_id');
			$pageData['categories'] = $this->mcategory->getAllCategories();
			
			$this->load->view('header', $pageData);
			$this->load->view('categories');
			$this->load->view('addcommentform');
			$this->load->view('footer');
		}
		else {
			//форма заполнена правильно
			$this->load->library('htmlpurifier/HTMLPurifier');
			$config = HTMLPurifier_Config::createDefault();
			
			$commentData['uname'] = $this->htmlpurifier->purify($this->input->post('uname'));
			$commentData['uemail'] = $this->input->post('email');
			$commentData['description'] = $this->htmlpurifier->purify($this->input->post('description'));
			$commentData['bug_id'] = $this->input->post('bug_id');
			$commentData['parent_id'] = $this->input->post('parent_id');
			if (empty($commentData['parent_id'])) {
				$commentData['parent_id'] = null;
			}
			
			if ((($res = $this->mcomment->addNewComment($commentData)) !== FALSE)
				&& (is_integer((int)$commentData['bug_id']))
				&& (is_integer((int)$commentData['parent_id']) || $commentData['parent_id'] == null)) {
				//данные добавлены
				$this->session->set_flashdata('message', 'Комментарий добавлен');
			}
			else {
				//при добавлении данных возникла ошибка
				$this->session->set_flashdata('message', 'При добавлении комментария возникла ошибка');
			}
			redirect($this->session->userdata('prev_page'));
		}
	}
	
	/**
	 * Формирует страницу с формой для ввода имени и пароля
	 * @return html страница, если посетитель уже прошел аутентификацию - редирект
	 * на главную страницу
	 */
	function login() {
		if ($this->redux_auth->logged_in()) {
			redirect('bugtracker/page');
		}
    	$pageData['title'] = 'Login';
    	$pageData['categories'] = $this->mcategory->getAllCategories();

    	$this->load->view('header', $pageData);
		$this->load->view('categories');
        $this->load->view('login');
        $this->load->view('footer');
	}
	
	/**
	 * Выполняет аутентификацию постетителя. Проверяет введенные email и пароль.
	 * @return в случае успеха - редирект на главную,
	 * в случае неудачи - html страница с формой и описаниями ошибок. 
	 */
	function checklogin() {
		$this->load->library('form_validation');
		$this->form_validation->set_rules('email', 'lang:email', 'required|valid_email');
	    $this->form_validation->set_rules('password', 'lang:password', 'required');
	    $this->form_validation->set_error_delimiters('<div class="fErrMessage">', '</div>');
	    
	    if ($this->form_validation->run() == false)
	    {
	    	$pageData['title'] = 'Login error';
	        $this->load->view('header', $pageData);
	        $this->load->view('login');
	        $this->load->view('footer');
	    }
	    else
	    {
	        $login    = $this->input->post('email');
	        $password = $this->input->post('password');
	        
	        if ($this->redux_auth->login($login, $password)) {
	        	$userData = $this->redux_auth->profile();
	        	$this->session->set_flashdata('message', 'Привет, '.$userData->username);
	        	redirect('bugtracker/page');
	        }
	        else {
	        	$this->session->set_flashdata('message', 'Неверная пара логин/пароль, попробуйте ещё раз');
		        redirect('bugtracker/login');
	        }
	    }
	}
	
	/**
	 * Выход из администраторского режима
	 * @return редирект на главную
	 */
	function logout()
	{
		$this->redux_auth->logout();
		$this->session->set_flashdata('message', 'Вы вышли из администраторского режима');
		redirect('bugtracker/page');
	}
	
	/**
	 * Этот метод используется для создания администраторского аккаунта.
	 * В параметрах метода register нужно указать:
	 * 1) ник;
	 * 2) пароль;
	 * 3) email (логин).
	 * После создания аккаунта метод нужно закомментировать,
	 * иначе аккаунт сможет создать кто угодно.
	 */
//	function register() {
//		echo $this->redux_auth->register('vova', '111', 'vvv@ddd.sss');
//	}
	
	/**
	 * Удаляет указанную запись о баге. Перед выполнением операции выполняется
	 * авторизация пользователя.
	 * @param $bugId - id бага в базе данных
	 * @return в случае успеха - редирект на последнюю страницу с которой был
	 * отправлен запрос на удаление, в случае неудачи - редирект на последнюю
	 * страницу с которой был отправлен запрос на удаление с сообщением об
	 * ошибке, в случае отсутствия у пользователя соответствующих прав - редирект
	 * на страницу с формой входа.
	 */
	function deletebug($bugId) {
		if ($this->redux_auth->logged_in()) {
			if ($this->mbug->delete($bugId)) {
				$this->session->set_flashdata('message', 'Баг удален');
				redirect($this->session->userdata('prev_page'));
			}
			else {
				$this->session->set_flashdata('message', 'При удалении бага возникла ошибка');
				redirect($this->session->userdata('prev_page'));
			}
		}
		else {
        	$this->session->set_flashdata('message', 'Вы должны войти для того, чтобы выполнить это действие');
	        redirect('bugtracker/login');
		}
	}
	
	/**
	 * Удаляет указанный комментарий. Перед выполнением операции выполняется
	 * авторизация пользователя.
	 * @param $commentId - id комментария в базе данных
	 * @return в случае успеха - редирект на последнюю страницу с которой был
	 * отправлен запрос на удаление, в случае неудачи - редирект на последнюю
	 * страницу с которой был отправлен запрос на удаление с сообщением об
	 * ошибке, в случае отсутствия у пользователя соответствующих прав - редирект
	 * на страницу с формой входа.
	 */
	function deletecomment($commentId) {
		if ($this->redux_auth->logged_in()) {
			if ($this->mcomment->delete($commentId)) {
				$this->session->set_flashdata('message', 'Комментарий удален');
				redirect($this->session->userdata('prev_page'));
			}
			else {
				$this->session->set_flashdata('message', 'При удалении комментария возникла ошибка');
				redirect($this->session->userdata('prev_page'));
			}
		}
		else {
        	$this->session->set_flashdata('message', 'Вы должны войти для того, чтобы выполнить это действие');
	        redirect('bugtracker/login');
		}
	}
	
	/**
	 * Создает страницу с формой добавления новой категории. Предварительно
	 * выполняется авторизация пользователя.
	 * @return html страница с формой - если авторизация пройдена, в противном
	 * случае - редирект на страницу с формой входа.
	 */
	function newcategory() {
		if (!$this->redux_auth->logged_in()) {
        	$this->session->set_flashdata('message', 'Вы должны войти для того, чтобы выполнить это действие');
	        redirect('bugtracker/login');
	        return;
		}
		//сохраняем адрес этой страницы в сессии
		$this->session->set_userdata(array('prev_page'=>current_url()));
		
		$pageData['title'] = 'Bugtracker | New Category';
		$pageData['categories'] = $this->mcategory->getAllCategories();
		
		$this->load->view('header', $pageData);
		$this->load->view('categories');
		$this->load->view('addcategoryform');
		$this->load->view('footer');
	}
	
	/**
	 * Создает новую категорию. Данные передаются в массиве $_POST.
	 * Предварительно выполняется авторизация пользователя.
	 * @return в случае успеха - редирект на предыдущую страницу,
	 * в случае неудачи - html страница с формой и описаниями ошибок,
	 * если авторизация не пройдена - редирект на страницу с формой входа. 
	 */
	function addcategory() {
		if (!$this->redux_auth->logged_in()) {
        	$this->session->set_flashdata('message', 'Вы должны войти для того, чтобы выполнить это действие');
	        redirect('bugtracker/login');
	        return;
		}
		$this->load->library('form_validation');
		$this->form_validation->set_error_delimiters('<div class="fErrMessage">', '</div>');
		$this->form_validation->set_rules('categoryname', 'lang:categoryname', 'required');
		$this->form_validation->set_rules('categorylink', 'lang:categorylink', 'required|alpha_dash');
		if ($this->form_validation->run() === FALSE) {
			//ошибка
			$pageData['message'] = 'Форма заполнена неправильно';
			
			//показываем форму с описаниями ошибок
			$pageData['title'] = 'Bug Tracker | Add Category';
			$pageData['categories'] = $this->mcategory->getAllCategories();
			
			$this->load->view('header', $pageData);
			$this->load->view('categories');
			$this->load->view('addcategoryform');
			$this->load->view('footer');
		}
		else {
			//форма заполнена правильно
			$this->load->library('htmlpurifier/HTMLPurifier');
			$config = HTMLPurifier_Config::createDefault();
			
			$categoryData['name'] = $this->htmlpurifier->purify($this->input->post('categoryname'));
			$categoryData['link'] = $this->input->post('categorylink');
			
			if (($res = $this->mcategory->addCategory($categoryData)) !== FALSE) {
				//данные добавлены
				$this->session->set_flashdata('message', 'Категория создана');
			}
			else {
				//при добавлении данных возникла ошибка
				$this->session->set_flashdata('message', 'При добавлении категории возникла ошибка');
			}
			redirect($this->session->userdata('prev_page'));
		}
	}
}
?>